#!/usr/bin/perl
#
# ======================================================
#   CONNTRACK-VIEWER version 1.3
#   http://cv.intellos.net
#
#   by Patrick Lagace patou@sympatico.ca
#
# ======================================================
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###2004-0319 - conntrack-viewer.pl v.0.3 modded for use with Connview v.0.3 by catastrophe
# - 'Read in Ethernet Settings' and 'Truncation' inspired by conntrack-viewer-KH.pl modifications 
#    by Venables
# - Added colored support. IP addresses/name resolution display in green, orange, red, and white. 
#    Ports display in gray
# - Added icmp interpretation section to tame icmp output, ports separated into their own columns
# - Broke code off into subroutines so I could read it easier
# - Implemented "dumb" forward/reverse sorting by column utilizing a temporary file
###2004-0403 - conntrack-viewer-CAT.pl v.0.4 modded for use with Connview v.0.4 by catatstrophe
# - Renamed modified conntrack-viewer.pl to conntrack-viewer-CAT.pl
# - Cleaned up output, fixed 'tab' glitch that shifted out with MSIE6
# - 'Smart' IP sorting using ip2number 'pre-sort' routine from DHCP Pack v1.2 (dhcplease.pl 
#    by Dane Robert Jones and Tiago Freitas Leal)
# - 'Smart' port sorting by enabling '-n' during sorting
###2004-0505 - conntrack-viewer-CAT.pl v.0.5 modded for use with Connview v.0.5 by catatstrophe
# - Reworked display into using 'html tables' rather than using 'spacing' (with name resolution 
#    each entry takes two lines, state may be represented with graphic)
# - When Sort by is 'None', temporary sorting file no longer generated.
# - Added "Connection summary" section
# - Added "Views" which allows for Both, Masq, Direct, and Mixed viewing modes
# - Unknown protocol support and no port no service check sections by JKS
# - Raw /proc/net/ip_conntrack output section added
# - Fixed some variable initialization problems
# - Source and remote ip checkboxes for use with "Lookup" and "Add to IP block list" from 
#    firewalllog.dat by The Smoothwall Team
# - Enlarged font for IP addresses and ports for easy viewing on high resolution monitors
# - Graceful handling of undefined protocols
# - Hide UDP, TCP, or Other connection entries
###2004-0919 - conntrack-viewer-CAT.pl v.0.6 modded for use with Connview v.0.6 by catatstrophe
# - Fixed incorrect "protocol name" displayed during 'unknown' protocol interpretation
# - Incorporated "Cutter support" code by Daniel Hokka Zakrisson (daniel_hozac) 
#    to "cut" specific connection(s) based on "ip-address-1 port-1 ip-address-2 port-2"
# - Added "Lookup" and "Add to IP block list" code updates by Daniel Hokka Zakrisson (daniel_hozac)
# - Data gathering from ip_conntrack done by helper file to overcome "permission change" introduced 
#    by kernel 2.4.27 in Fixes 4 (Thanks SwampKracker, daniel_hozac, & NetWhiz)
# - Fixed "/bin/sort: field number is zero: invalid field specification `0.1'" bug
# - Truncation of resolved names from the left so the TLD can be viewed
# - Truncation of port names as well when truncation checkbox ticked
# - Fixed parsing problems for TCP & UDP because of extra "mark=0" with SK 2.4.27 (Thanks lucasl & foxhound)
# - Fixed parsing problems for ICMP (and others) because of extra "mark=0" with SK 2.4.27
# - Incorporated 2004-09-03 "modsys packaging and other smaller fixes" by Daniel Hokka Zakrisson (daniel_hozac)
#    (includes finalized protocol interpretation fix for SK 2.4.27)
# - Check if Cutter exists, if so then show Cutter button
# - Incorporated 2004-0919 "modys fixes" by Daniel Hokka Zakrisson (includes finalized Cutter "auto-detect")
###2005-0616 - conntrack-viewer-CAT.pl v.0.7 modded for use with Connview v.0.7 by catatstrophe
# - Added gre pptp protocol interpretation with SK 2.4.30 compatibility (thanks BoHiCa)
# - Connection summary for TCP now includes "mouse-over" showing 'established' connections
# - Tweaked so compatible with sylvester's blackcurrant color mod
# - Added "Summary Only", "Raw Only", and "Netstat" views
# - Incorporated "in script sorting" code contribution by Daniel Hokka Zakrisson (daniel_hozac)
# - 'Unified' protocol interpretation & other tweaks to substantially improve "in script sorting" performance
#
#
# - Disclaimer:
#    - Use at your own risk without any or implied warranty.
###





############
###HEADER###
############

require '/var/smoothwall/header.pl';

use Socket;

###############
###VARIABLES###
###############

###Read in Smoothwall ethernet settings
my %thisnetwork;
&readhash("${swroot}/ethernet/settings", \%thisnetwork);
$thisnetwork{'GREEN_NETADDRESS'} =~ /.0$/; $GREEN_NET = $`;
$thisnetwork{'ORANGE_NETADDRESS'} =~ /.0$/; $ORANGE_NET = $`;
$thisnetwork{'RED_NETADDRESS'} =~ /.0$/; $RED_NET = $`;
$thisnetwork{'BLUE_NETADDRESS'} =~ /.0$/; $BLUE_NET = $`;
$L0CALH0ST = "127.0.0.1";

###Read in Smoothwall red local-ipaddress
$redip = '0.0.0.0';
open(FILE, "${swroot}/red/local-ipaddress");
$redip = <FILE>; chomp $redip;
close(FILE);

###Read in Connview settings
#$conn_cutterfile = "/usr/local/bin/cutter";
$connviewsettings{'ACTION'} = '';
$connviewsettings{'CONN_DISPLAY'} = 'mixed';
$connviewsettings{'CONN_NAME'} = 'off';
$connviewsettings{'CONN_COLOR'} = 'off';
$connviewsettings{'CONN_TRUNCATE'} = 'off';
$connviewsettings{'CONN_SORT'} = 'on';
$connviewsettings{'CONN_COLUMN'} = '0';
$connviewsettings{'CONN_REFRESH'} = '0';
$connviewsettings{'CONN_HIDETCP'} = 'off';
$connviewsettings{'CONN_HIDEUDP'} = 'off';
$connviewsettings{'CONN_HIDEOTHER'} = 'off';
$connviewsettings{'CONN_REVERSE'} = 'off';
$connviewsettings{'CONN_RAW'} = 'off';
$connviewsettings{'CONN_CUTTER'} = 'on';
$connviewsettings{'CONN_TRUNCATE_LEFT'} = 'on';
&readhash("${swroot}/preferences/connview", \%connviewsettings);
$connviewsettings{'CONN_CUTTER'} = (-x "/usr/local/bin/cutter" && $connviewsettings{'CONN_CUTTER'} ne 'off' ? 'on' : 'off');

###Connview Counters
$connMASQcount=0;
$connDIRECTcount=0;
$connTOTALcount=0;
$connMASQcountESTABLISHED=0;
$connDIRECTcountESTABLISHED=0;
$connTOTALcountESTABLISHED=0;
$connMASQcountTCP=0;
$connDIRECTcountTCP=0;
$connTOTALcountTCP=0;
$connMASQcountUDP=0;
$connDIRECTcountUDP=0;
$connTOTALcountUDP=0;
$connMASQcountICMP=0;
$connDIRECTcountICMP=0;
$connTOTALcountICMP=0;
$connMASQcountOTHER=0;
$connDIRECTcountOTHER=0;
$connTOTALcountOTHER=0;

$connviewTICKETcount=1;

###Default Variables
if ($connviewsettings{'CONN_NAME'} eq "on") { $dnsresolution=1; } else { $dnsresolution=0; }
$masqueradeonly=0;
$directonly=0;
$showheader=1;

@ip_conntrack_brut = ();
@ip_conntrack_sort = ();

##########
###MAIN###
##########

if ($connviewsettings{'CONN_DISPLAY'} eq "netstat") { 
	&connviewNETSTATshow();

	if (($connviewsettings{'CONN_RAW'} eq "on") || ($connviewsettings{'CONN_DISPLAY'} eq "raw")) {
		&conntrackCONTENTget();
		&connviewRAWdisplay();
	}
} elsif ($connviewsettings{'CONN_DISPLAY'} eq "raw") { 
	&conntrackCONTENTget();
	&connviewRAWdisplay();
} elsif ($connviewsettings{'CONN_DISPLAY'} eq "summary") {
	&conntrackCONTENTget();

	for ($i=0; $i <= $#ip_conntrack_brut; $i++) {
		$proto = $ip_conntrack_brut[$i];

		&conntrackPROTOCOLinterpretation();	###protocol interpretation only once for summary view
		&connviewGOcount();
	}	
	&connviewCOUNTdisplay();				###count connections only once for summary view
	&connviewRAWdisplay();
} else {
	&connviewSWITCHread();
	&conntrackCONTENTget();
	&conntrackHEADERshow();

	for ($i=0; $i <= $#ip_conntrack_brut; $i++) {
		$proto = $ip_conntrack_brut[$i];

		&conntrackPROTOCOLinterpretation();	###protocol interpretation only once for both, masq, direct, and mixed views
		&connviewGOcount();				###count connections only once for both, masq, direct, and mixed views

		$ip2numbersrcaddr = &ip2number($srcaddr);
		$ip2numberdstaddr = &ip2number($dstaddr);
		$ip_conntrack_sort[$i] = "0,$proto,$srcaddr,$srcport,$dstaddr,$dstport,$portname,$state,$mornot,$ip2numbersrcaddr,$ip2numberdstaddr,$srcportname,$dstportname";
	}

	if ($connviewsettings{'CONN_SORT'} eq "on") {	###in script sorting
		@ip_conntrack_sort = sort { &conntrackCOMPARE(); } @ip_conntrack_sort;
	}

	for ($i=0; $i <= $#ip_conntrack_sort; $i++) {			
		&connviewPOSTSORT();
		&conntrackRESULTSdisplay(); 
	}
	$connviewTICKETcount=$connviewTICKETcount-1;

	if ($connviewTICKETcount eq 1) {
		$masqueradeonly=0;
		$directonly=1;

		print "</table>";

		&closebox();

		&conntrackHEADERshow();

		for ($i=0; $i <= $#ip_conntrack_sort; $i++) {	
			&connviewPOSTSORT();
			&conntrackRESULTSdisplay();
		}	
	}
	print "</table>";

	&closebox();

	&connviewGObuttons();
	&connviewCOUNTdisplay();
	&connviewRAWdisplay();
}

##########
###Done###
##########






#################
###SUBROUTINES###
#################

sub connviewSWITCHread {
	if ($connviewsettings{'CONN_DISPLAY'} eq "mixed") { 
		$masqueradeonly=0; 
		$directonly=0;
	} elsif ($connviewsettings{'CONN_DISPLAY'} eq "masq") { 
		$masqueradeonly=1; 
		$directonly=0;
	} elsif ($connviewsettings{'CONN_DISPLAY'} eq "direct") { 
		$masqueradeonly=0; 
		$directonly=1;
	} elsif ($connviewsettings{'CONN_DISPLAY'} eq "both") { 
		$masqueradeonly=1; 
		$directonly=0;
		$connviewTICKETcount=$connviewTICKETcount+1;
	} elsif ($connviewsettings{'CONN_DISPLAY'} eq "summary") { 
		$masqueradeonly=0; 
		$directonly=0;
	} elsif ($connviewsettings{'CONN_DISPLAY'} eq "netstat") { 
		###then do nothing
	}
}


sub conntrackCONTENTget {
	@ip_conntrack_brut = `/usr/local/bin/connviewhelper`;
#	@ip_conntrack_brut = `$sudo ${swroot}/modsys/helper.d/connview`;	###modsys
#	@ip_conntrack_brut = `cat /tmp/connviewDEBUG`;				###debugging
#	@ip_conntrack_brut = `cat /tmp/connviewDEBUGlarge10000`;		###debugging
#	@ip_conntrack_brut = `cat /tmp/connviewDEBUGlarge5000`;		###debugging
#	@ip_conntrack_brut = `cat /tmp/connviewDEBUGlarge3000`;		###debugging
#	@ip_conntrack_brut = `cat /tmp/connviewDEBUGlarge2000`;		###debugging
#	@ip_conntrack_brut = `cat /tmp/connviewDEBUGlarge1000`;		###debugging
#	@ip_conntrack_brut = `cat /tmp/connviewDEBUGlarge100`;			###debugging
#	@ip_conntrack_brut = `cat /tmp/connviewDEBUG2m1nm`;			###debugging
}


sub conntrackHEADERshow {
	if ($showheader){
		if ($connviewsettings{'CONN_DISPLAY'} eq "mixed") { 
			&openbox('100%', 'LEFT', $tr{'conn active mixed connections'});
		} elsif (($connviewsettings{'CONN_DISPLAY'} eq "masq") || ($connviewsettings{'CONN_DISPLAY'} eq "both") && ($connviewTICKETcount eq 2))  {
			&openbox('100%', 'LEFT', $tr{'conn active masqueraded connections'});
		} elsif (($connviewsettings{'CONN_DISPLAY'} eq "direct") || ($connviewsettings{'CONN_DISPLAY'} eq "both") && ($connviewTICKETcount eq 1)) { 
			&openbox('100%', 'LEFT', $tr{'conn active direct connections'});
		} 
		print "<table width='100%' border='0' cellspacing='2' cellpadding='0'>";
   		print "<TR>";
		print "<TD width='6%' align='left'><tt>Proto</tt></TD>";
   		print "<TD width='10%' align='center'><tt>State</tt></TD>";
		print "<TD>&nbsp;</TD>";
		print "<TD width='27%' align='left'><tt>Source IP</tt></TD>";
		print "<TD width='10%' align='left'><tt>Port</tt></TD>";
		print "<TD>&nbsp;</TD>";
   		print "<TD width='27%' align='left'><tt>Remote IP</tt></TD>";
		print "<TD width='10%' align='left'><tt>Port</tt></TD>";
    		print "</TR>\n";
 	}
}


sub conntrackPROTOCOLinterpretation {
	$proto =~ s/\[\S+\]\s//;

	for ($proto){
		if (/tcp/){
			###tcp 6 426032 ESTABLISHED src=127.0.0.1 dst=127.0.0.1 sport=1998 dport=1997 src=127.0.0.1 dst=127.0.0.1 sport=1997 dport=1998 [ASSURED] use=1
			###tcp 6 431968 ESTABLISHED src=155.0.x.x dst=69.10.xxx.xxx sport=1478 dport=80 src=155.0.x.x dst=155.0.x.x sport=800 dport=1478 [ASSURED] use=1 mark=0
			$proto =~ s/(\w+)\s+(\d+)\s(\S+)\s(\w+)\ssrc=(\S+)\sdst=(\S+)\ssport=(\S+)\sdport=(\S+)\s+src=(\S+)\sdst=(\S+)\ssport=(\S+)\sdport=(\S+)\s+use=(\S+)(\smark=\d+)?\s*\n/$1/;
#####			$proto =~ /(\w+)\s+(\d+)\s(\S+)\s(\w+)\ssrc=(\S+)\sdst=(\S+)\ssport=(\S+)\sdport=(\S+)\s+src=(\S+)\sdst=(\S+)\ssport=(\S+)\sdport=(\S+)\s+use=(\S+)(\smark=\d+)?\s*\n/;
			$state	= $4; $srcaddr = $5; $dstaddr = $6; $srcport = $7; $dstport = $8; $mornot = $10;
		} elsif (/udp/) {
			###udp 17 26 src=68.40.xxx.xxx dst=255.255.255.255 sport=67 dport=68 [UNREPLIED] src=255.255.255.255 dst=68.40.xxx.xxx sport=68 dport=67 use=1
			###udp 17 174 src=127.0.0.1 dst=127.0.0.1 sport=1027 dport=53 src=127.0.0.1 dst=127.0.0.1 sport=53 dport=1027 [ASSURED] use=1 mark=0
			$proto =~ s/(\w+)\s+(\d+)\s(\S+)\ssrc=(\S+)\sdst=(\S+)\ssport=(\S+)\sdport=(\S+)\s+src=(\S+)\sdst=(\S+)\ssport=(\S+)\sdport=(\S+)\s+use=(\S+)(\smark=\d+)?\s*\n/$1/;
#####			$proto =~ /(\w+)\s+(\d+)\s(\S+)\ssrc=(\S+)\sdst=(\S+)\ssport=(\S+)\sdport=(\S+)\s+src=(\S+)\sdst=(\S+)\ssport=(\S+)\sdport=(\S+)\s+use=(\S+)(\smark=\d+)?\s*\n/;
			$state = " "; $srcaddr = $4; $dstaddr = $5; $srcport = $6; $dstport = $7; $mornot = $9;
		} elsif (/icmp/) {
			###icmp 1 26 src=192.168.1.102 dst=192.168.1.101 type=8 code=0 id=512 [UNREPLIED] src=192.168.1.101 dst=192.168.1.102 type=0 code=0 id=512 use=1
			###icmp 1 29 src=192.168.100.1 dst=82.255.xx.xxx type=8 code=0 id=512 src=82.255.xx.xxx dst=82.64.xxx.xxx type=0 code=0 id=512 use=1 mark=0
			$proto =~ s/(\w+)\s+(\d+)\s(\S+)\ssrc=(\S+)\sdst=(\S+)\stype=(\S+)\scode=(\S+)\sid=(\S+)\s+src=(\S+)\sdst=(\S+)\stype=(\S+)\scode=(\S+)\sid=(\S+)\s+use=(\S+)(\smark=\d+)?\s*\n/$1/;
#####			$proto =~ /(\w+)\s+(\d+)\s(\S+)\ssrc=(\S+)\sdst=(\S+)\stype=(\S+)\scode=(\S+)\sid=(\S+)\s+src=(\S+)\sdst=(\S+)\stype=(\S+)\scode=(\S+)\sid=(\S+)\s+use=(\S+)(\smark=\d+)?\s*\n/;
			$state = " "; $srcaddr = $4; $dstaddr = $5; $srcport = "-"; $dstport = "-"; $mornot = $10; ; $portname=" ";
		} elsif (/EXPECTING:/) {
			###EXPECTING: - use=1 proto=6 src=146.xxx.xxx.xxx dst=RED.xxx.xxx.xxx sport=0 dport=1199
			$proto =~ s/(\w+):\s(\S+)\s+use=(\S+)\sproto=(\S+)\ssrc=(\S+)\sdst=(\S+)\ssport=(\S+)\sdport=(\S+)(\smark=\d+)?\s*\n/$1/;
#####			$proto =~ /(\w+):\s(\S+)\s+use=(\S+)\sproto=(\S+)\ssrc=(\S+)\sdst=(\S+)\ssport=(\S+)\sdport=(\S+)(\smark=\d+)?\s*\n/;
			$state = "&quot;$1&quot;"; $srcaddr = $5; $dstaddr = $6; $srcport = $7; $dstport = $8; $mornot = " ";

			unless ($proto = getprotobynumber $4) { $proto = $4; }
		} elsif (/gre/) {
			###gre 47 431999 timeout=600, stream_timeout=432000 src=remote.pptp.server.xxx dst=RED.xxx.xxx.xxx version=1 protocol=0x880b srckey=0x0000 dstkey=0x000 src=GRN.end.point.xxx dst=remote.pptp.server.xxx version=1 protocol=0x880b srckey=0x0 dstkey=0x0000 [ASSURED] use=11
			$proto =~ s/(\w+)\s+(\d+)\s(\S+)\stimeout=(\S+),\sstream_timeout=(\S+)\ssrc=(\S+)\sdst=(\S+)\sversion=(\S+)\sprotocol=(\S+)\ssrckey=(\S+)\sdstkey=(\S+)\s+src=(\S+)\s+dst=(\S+)\s+version=(\S+)\s+protocol=(\S+)\s+srckey=(\S+)\s+dstkey=(\S+)\s+use=(\S+)(\smark=\d+)?\s*\n/$1/;
			$state = " "; $srcaddr = $12; $dstaddr = $6; $srcport = "-"; $dstport = "-"; $mornot = $7; 

			if ($srcaddr eq "") {	###for SK30 compatibility
			###gre 47 431979 timeout=600, stream_timeout=432000 src=63.xxx.xxx.xxx dst=24.xxx.xxx.xxx srckey=0x8c00 dstkey=0x1000 src=192.168.1.100 dst=63.xxx.xxx.xxx srckey=0x8000 dstkey=0x8c00 [ASSURED] use=2 mark=0
			$proto =~ s/(\w+)\s+(\d+)\s(\S+)\stimeout=(\S+),\sstream_timeout=(\S+)\ssrc=(\S+)\sdst=(\S+)\ssrckey=(\S+)\sdstkey=(\S+)\s+src=(\S+)\s+dst=(\S+)\s+srckey=(\S+)\s+dstkey=(\S+)\s+use=(\S+)(\smark=\d+)?\s*\n/$1/;
			$state = " "; $srcaddr = $10; $dstaddr = $11; $srcport = "-"; $dstport = "-"; $mornot = $7;
			}
		} elsif (/unknown/) {
			###unknown  47 599 src=192.168.0.xxx dst=xxx.xxx.xxx.xxx src=xxx.xxx.xxx.xxx dst=RED.xxx.xxx.xxx use=1
			$proto =~ s/(\w+)\s+(\d+)\s(\S+)\ssrc=(\S+)\sdst=(\S+)\s+src=(\S+)\sdst=(\S+)\s+use=(\S+)(\smark=\d+)?\s*\n/$1/;
#####			$proto =~ /(\w+)\s+(\d+)\s(\S+)\ssrc=(\S+)\sdst=(\S+)\s+src=(\S+)\sdst=(\S+)\s+use=(\S+)(\smark=\d+)?\s*\n/;
			$state = " "; $srcaddr = $4; $dstaddr = $5; $srcport = "-"; $dstport = "-"; $mornot = $7;

			unless ($proto = getprotobynumber $2) { $proto = $2; }
		} else {
			$proto = "[???]";
			$state = " "; $srcaddr = " "; $dstaddr = " "; $srcport = " "; $dstport = " "; $mornot = " ";
		}

		$srcportname="[???]";
		$dstportname="[???]";

		if ($srcport ne " " && $srcport ne "-") { unless ($srcportname = getservbyport $srcport,$proto) { $srcportname = "[???]"; } }
		if ($dstport ne " " && $dstport ne "-") { unless ($dstportname = getservbyport $dstport,$proto) { $dstportname = "[???]"; } }

		if ($srcportname eq "[???]" && $dstportname eq "[???]") { $portname="[???]"; }
		if ($srcportname ne "[???]" && $dstportname eq "[???]") { $portname=$srcportname; }
		if ($srcportname eq "[???]" && $dstportname ne "[???]") { $portname=$dstportname; }
		if ($srcportname ne "[???]" && $dstportname ne "[???]") { $portname=$srcportname."-".$dstportname; }
 	}
}


sub ip2number {
	my $ip = $_[0];
	if (!($ip =~ /^(\d+)\.(\d+)\.(\d+)\.(\d+)$/)) {
		return 0; 
	} else {
		return ($1*(256*256*256))+($2*(256*256))+($3*256)+$4; 
	}
}


sub conntrackCOMPARE {	###in script sorting (compare two conntracks)
	my (@one, @two);
	my $ret;

	$yeah = $a;
	@one = split(/\,/,$yeah);

	$yeah = $b;
	@two = split(/\,/,$yeah);

	$ret = $one[$connviewsettings{'CONN_COLUMN'}] <=> $two[$connviewsettings{'CONN_COLUMN'}];
	# <=> returns undef when the operands aren't numbers
	if ($ret == undef)
	{
		# use lexical comparison instead
		$ret = $one[$connviewsettings{'CONN_COLUMN'}] cmp $two[$connviewsettings{'CONN_COLUMN'}];
	}

	# invert
	if ($connviewsettings{'CONN_REVERSE'} eq "on")
	{
		$ret = -$ret;
	}
 
	return $ret;
}


sub connviewPOSTSORT {
		$yeah = $ip_conntrack_sort[$i];

		chomp($yeah);
		my @sorted = split(/\,/,$yeah);

		$proto = $sorted[1]; 
		$srcaddr = $sorted[2]; 
		$srcport = $sorted[3];
		$dstaddr = $sorted[4]; 
		$dstport = $sorted[5];  
		$portname = $sorted[6];  
		$state = $sorted[7]; 
		$mornot = $sorted[8]; 
		$ip2numbersrcaddr = $sorted[9]; 
		$ip2numberdstaddr = $sorted[10]; 
		$srcportname = $sorted[11]; 
		$dstportname = $sorted[12]; 
}


sub conntrackRESULTSdisplay {
	if (($connviewsettings{'CONN_HIDEUDP'} eq "on") && ($proto eq "udp")) { ###then do nothing 
	} elsif (($connviewsettings{'CONN_HIDETCP'} eq "on") && ($proto eq "tcp")) { ###then do nothing 
	} elsif (($connviewsettings{'CONN_HIDEOTHER'} eq "on") && ($proto ne "udp") && ($proto ne "tcp")) { ###then do nothing 
	} elsif (($masqueradeonly && ($srcaddr ne $mornot)) || (!$masqueradeonly && !$directonly)){     
		&connviewMAIN_COLORED();
		&conntrackDNSdisplay();
	} elsif ($directonly && ($srcaddr eq $mornot)) {
		&connviewMAIN_COLORED();
		&conntrackDNSdisplay();
	}
}


sub connviewMAIN_COLORED {
	if ($connviewsettings{'CONN_COLOR'} eq "on") {
		print "<TR>";
		print "<TD><tt>$proto</tt></TD>";

		&connviewSTATEshow();

		print "<TD><INPUT TYPE='checkbox' NAME='conn' VALUE='$srcaddr:$srcport/$dstaddr:$dstport(s)'></TD>";

		$colormenow1=$srcaddr; $colormenow2=$srcaddr; &connviewCOLORmeNOWbig();

		if (!$masqueradeonly && !$directonly) { if ($srcaddr ne $mornot) { print " M"; } }

		print "</TD>";
		
		print qq|
		<TD style='cursor: pointer' title="$srcportname" CLASS='base' ALIGN='left'><font color=#666666>$srcport</font></TD>
		|;

		print "<TD><INPUT TYPE='checkbox' NAME='conn' VALUE='$srcaddr:$srcport/$dstaddr:$dstport(d)'></TD>";

		$colormenow1=$dstaddr; $colormenow2=$dstaddr; &connviewCOLORmeNOWbig();

		print qq|
		<TD style='cursor: pointer' title="$dstportname" CLASS='base' ALIGN='left'><font color=#666666>$dstport</font></TD>
		|;
	} else {
		print "<TR>";
		print "<TD><tt>$proto</tt></TD>";

		&connviewSTATEshow();

		print "<TD><INPUT TYPE='checkbox' NAME='conn' VALUE='$srcaddr:$srcport/$dstaddr:$dstport(s)'></TD>";
		print "<TD>$srcaddr";

		if (!$masqueradeonly && !$directonly) { if ($srcaddr ne $mornot) { print " M"; } }

		print "</TD>";

		print qq|
		<TD style='cursor: pointer' title="$srcportname" onClick="javascript:;" CLASS='base' ALIGN='left'>$srcport</TD>
		|;

		print "<TD><INPUT TYPE='checkbox' NAME='conn' VALUE='$srcaddr:$srcport/$dstaddr:$dstport(d)'></TD>";
		print "<TD>$dstaddr</TD>";

		print qq|
		<TD style='cursor: pointer' title="$dstportname" CLASS='base' ALIGN='left'>$dstport</TD>
		|;
	}
	print "</TR>";
}


sub connviewSTATEshow {
	if ($state eq "ESTABLISHED") {
		print qq|
		<TD style='cursor: pointer' title="ESTABLISHED" CLASS='base' ALIGN='center'><IMG SRC='/ui/assets/3.5/img/on.gif'></TD>
		|;
	} elsif ($state eq "CLOSE") {
		print qq|
		<TD style='cursor: pointer' title="CLOSE" CLASS='base' ALIGN='center'><IMG SRC='/ui/assets/3.5/img/off.gif'></a></TD>
		|;
	} elsif ($state eq "CLOSE_WAIT") {
		print "<TD ALIGN='center'><TT>$state</TT></TD>";
	} elsif ($state eq "TIME_WAIT") {
		print "<TD ALIGN='center'><TT>$state</TT></TD>";
	} elsif ($state eq "SYN_RECV") {
		print "<TD ALIGN='center'><TT>$state</TT></TD>";
	} elsif ($state eq "-" && $proto eq "udp") {
		print qq|
		<TD style='cursor: pointer' title="" CLASS='base' ALIGN='center'>-</a></TD>
		|;
	} else {
		print "<TD ALIGN='center'><TT>$state</TT></TD>";
	}
}


sub conntrackDNSdisplay {

	if ($srcportname eq "[???]") { $srcportname = " "; } 
	if ($dstportname eq "[???]") { $dstportname = " "; }

	if ($dnsresolution){
		$srcname = "&nbsp;";
		$dstname = "&nbsp;";

		if ($srcaddr ne " ") { if ($srcname = gethostbyaddr(inet_aton($srcaddr),AF_INET)) { } else { $srcname = "UNRESOLVED!"; } };
		if ($dstaddr ne " ") { if ($dstname = gethostbyaddr(inet_aton($dstaddr),AF_INET)) { } else { $dstname = "UNRESOLVED!"; } };

		if ($connviewsettings{'CONN_TRUNCATE'} eq "on") {	###truncate resolved names 
			if ($connviewsettings{'CONN_TRUNCATE_LEFT'} eq "on") {	
				if (length($srcname) > 30) { $srcname = "...".( substr $srcname, -27 ); };
				if (length($dstname) > 30) { $dstname = "...".( substr $dstname, -27 ); };
			} else {
				if (length($srcname) > 30) { $srcname = ( substr $srcname, 0, 27 )."..."; };
				if (length($dstname) > 30) { $dstname = ( substr $dstname, 0, 27 )."..."; };
			}
		}
		if ($connviewsettings{'CONN_COLOR'} eq "on") {	###color DNS display
			print "<TR><TD></TD><TD></TD><TD></TD>";

			$colormenow1=$srcaddr; $colormenow2=$srcname; &connviewCOLORmeNOWsmall();

			&connviewPORTNAMEtruncation();

			print "</TD>";
			print "<TD><TT>$srcportname</TT></TD>";
			print "<TD></TD>";

			$colormenow1=$dstaddr; $colormenow2=$dstname; &connviewCOLORmeNOWsmall();	
			
			print "</TD><TD><TT>$dstportname</TT></TD>";
			print "</TR>";
		} else {	###non-colored DNS display
			&connviewPORTNAMEtruncation();

			print "<TR><TD></TD><TD></TD><TD></TD>"; 
			print "<TD><TT>$srcname</TT></TD>";
			print "<TD><TT>$srcportname</TT></TD>";
			print "<TD></TD>";
			print "<TD><TT>$dstname</TT></TD>";
			print "<TD><TT>$dstportname</TT></TD>";
			print "</TR>";
		}
	} else {
		print "</TR>";
	}
}


sub connviewCOLORmeNOWbig {
	if ( $colormenow1 =~ /^$GREEN_NET/ ) {
		print "<TD><font color=#60aa60>$colormenow2</font>";
	} elsif ( $colormenow1 =~ /^$ORANGE_NET/ ) {
		print "<TD><font color=#ff9900>$colormenow2</font>";
	} elsif ( $colormenow1 =~ /^$RED_NET/ ) {
		print "<TD><font color=#ff6060>$colormenow2</font>";
	} elsif ( $colormenow1 =~ /^$redip/ ) { 
		print "<TD><font color=#ff6060>$colormenow2</font>";
	} elsif ( $colormenow1 =~ /^$L0CALH0ST/ ) { 
		print "<TD><font color=#ffffff>$colormenow2</font>";
	} elsif ( $colormenow1 =~ /^$BLUE_NET/ ) { 
		print "<TD><font color=#0000ff>$colormenow2</font>";
	} else {
		print "<TD>$colormenow2";
	}
}


sub connviewCOLORmeNOWsmall {
	if ( $colormenow1 =~ /^$GREEN_NET/ ) {
		print "<TD><tt><font color=#60aa60>$colormenow2</font></tt>";
	} elsif ( $colormenow1 =~ /^$ORANGE_NET/ ) {
		print "<TD><tt><font color=#ff9900>$colormenow2</font></tt>";
	} elsif ( $colormenow1 =~ /^$RED_NET/ ) {
		print "<TD><tt><font color=#ff6060>$colormenow2</font></tt>";
	} elsif ( $colormenow1 =~ /^$redip/ ) { 			
		print "<TD><tt><font color=#ff6060>$colormenow2</font></tt>";
	} elsif ( $colormenow1 =~ /^$L0CALH0ST/ ) { 
		print "<TD><tt><font color=#ffffff>$colormenow2</font></tt>";
	} elsif ( $colormenow1 =~ /^$BLUE_NET/ ) { 
		print "<TD><tt><font color=#0000ff>$colormenow2</font></tt>";
	} else {
		print "<TD><TT>$colormenow2</TT>";
	}
}


sub connviewPORTNAMEtruncation {
	if ($connviewsettings{'CONN_TRUNCATE'} eq "on") {	###truncate port names
		if (length($srcportname) > 11) { $srcportname = ( substr $srcportname, 0, 8 )."..."; };
		if (length($dstportname) > 11) { $dstportname = ( substr $dstportname, 0, 8 )."..."; };	
	}
}


sub connviewGOcount {
	if (($connviewTICKETcount eq 1) && ($connviewsettings{'CONN_DISPLAY'} ne "both") || ($connviewTICKETcount eq 2) && ($connviewsettings{'CONN_DISPLAY'} eq "both")) {
		$connTOTALcount=$connTOTALcount+1;

		if ($srcaddr ne $mornot) {
			$connMASQcount=$connMASQcount+1;

			if ($state eq "ESTABLISHED") { $connMASQcountESTABLISHED=$connMASQcountESTABLISHED+1; }

			if ($proto eq "tcp") { $connMASQcountTCP=$connMASQcountTCP+1; 
			} elsif ($proto eq "udp") { $connMASQcountUDP=$connMASQcountUDP+1; 
			} elsif ($proto eq "icmp") { $connMASQcountICMP=$connMASQcountICMP+1; 
			} else { $connMASQcountOTHER=$connMASQcountOTHER+1; } 
		} elsif ($srcaddr eq $mornot) {
			$connDIRECTcount=$connDIRECTcount+1;

			if ($state eq "ESTABLISHED") { $connDIRECTcountESTABLISHED=$connDIRECTcountESTABLISHED+1; }

			if ($proto eq "tcp") { $connDIRECTcountTCP=$connDIRECTcountTCP+1; 
			} elsif ($proto eq "udp") { $connDIRECTcountUDP=$connDIRECTcountUDP+1; 
			} elsif ($proto eq "icmp") { $connDIRECTcountICMP=$connDIRECTcountICMP+1; 
			} else { $connDIRECTcountOTHER=$connDIRECTcountOTHER+1; } 
		}
	}
}


sub connviewCOUNTdisplay {
	&openbox('100%', 'LEFT', $tr{'conn summaryc'});

	$connTOTALcountESTABLISHED=$connMASQcountESTABLISHED+$connDIRECTcountESTABLISHED;
	$connTOTALcountTCP=$connMASQcountTCP+$connDIRECTcountTCP;
	$connTOTALcountUDP=$connMASQcountUDP+$connDIRECTcountUDP;
	$connTOTALcountICMP=$connMASQcountICMP+$connDIRECTcountICMP;
	$connTOTALcountOTHER=$connMASQcountOTHER+$connDIRECTcountOTHER;

	print "<table width='100%' border='0' cellspacing='0' cellpadding='0'>";
	print "<TR>";
	print "<TD align='right'><TT>Masqueraded :  </TT></TD>";
	print "<TD align='right'><TT> $connMASQcount</TT></TD>";
	print "<TD><TT>&nbsp;connections</TT></TD>";
	print qq|
	<td style='cursor: pointer' title="$connMASQcountESTABLISHED / $connMASQcountTCP established"><TT>TCP: $connMASQcountTCP</TT></TD>
	|;
	print "<TD><TT>UDP: $connMASQcountUDP</TT></TD>";
	print "<TD><TT>ICMP: $connMASQcountICMP</TT></TD>";
	print "<TD><TT>OTHER: $connMASQcountOTHER</TT></TD>";
	print "</TR><TR>";
	print "<TD align='right'><TT>Direct :  </TT></TD>";
	print "<TD align='right'><TT> $connDIRECTcount</TT></TD>";
	print "<TD><TT>&nbsp;connections</TT></TD>";
	print qq|
	<td style='cursor: pointer' title="$connDIRECTcountESTABLISHED / $connDIRECTcountTCP established"><TT>TCP: $connDIRECTcountTCP</TT></TD>
	|;
	print "<TD><TT>UDP: $connDIRECTcountUDP</TT></TD>"; 
	print "<TD><TT>ICMP: $connDIRECTcountICMP</TT></TD>";
	print "<TD><TT>OTHER: $connDIRECTcountOTHER</TT></TD>";
	print "</TR><TR>";
	print "<TD align='right'><TT>Total :  </TT></TD>";
	print "<TD align='right'><TT> $connTOTALcount</TT></TD>";
	print "<TD><TT>&nbsp;connections</TT></TD>";
	print qq|
	<td style='cursor: pointer' title="$connTOTALcountESTABLISHED / $connTOTALcountTCP established"><TT>TCP: $connTOTALcountTCP</TT></TD>
	|;
	print "<TD><TT>UDP: $connTOTALcountUDP</TT></TD>";
	print "<TD><TT>ICMP: $connTOTALcountICMP</TT></TD>";
	print "<TD><TT>OTHER: $connTOTALcountOTHER</TT></TD>";
	print "</TR>";
	print "</TT></TD>";
	print "<TR></TABLE>";

	&closebox();
}


sub connviewGObuttons {
	&openbox('100%', 'LEFT');

	print qq|
	<table width='100%' border='0' cellspacing='0' cellpadding='0'>
	<TR>
		<TD WIDTH='10%' ALIGN='CENTER'><INPUT TYPE='submit' NAME='ACTION' VALUE='$tr{'lookup'}'></TD>
		<TD WIDTH='10%' ALIGN='CENTER'><INPUT TYPE='submit' NAME='ACTION' VALUE='$tr{'add to ip block'}'></TD>
	|;

	if ($connviewsettings{'CONN_CUTTER'} eq "on") {
		print qq|
		<TD WIDTH='10%' ALIGN='CENTER'><INPUT TYPE='submit' NAME='ACTION' VALUE='$tr{'conn cutter'}'></TD>
		|;
	}

	print "</TR></TABLE>";

	&closebox();
}


sub connviewRAWdisplay {
	if (($connviewsettings{'CONN_RAW'} eq "on") || ($connviewsettings{'CONN_DISPLAY'} eq "raw")) {
		&openbox('100%', 'LEFT', $tr{'conn rawc'});

		print "<table width='100%' border='0' cellspacing='2' cellpadding=''>";

		$v=0;

		for ($i=0; $i <= $#ip_conntrack_brut; $i++) {
			$raw = $ip_conntrack_brut[$i];

			if ($v eq $i) {
				print "<TR BGCOLOR=$table1colour><TD><TT><font color=#000000>$raw</FONT></TT></TD></TR>";
				$v=$v+2;
			} else {
				print "<TR BGCOLOR=$table2colour><TD><TT><font color=#000000>$raw</FONT></TT></TD></TR>";
			}
		}
		print "</table>";

		&closebox();
	}
}


sub connviewNETSTATshow {
	&openbox('100%', 'LEFT', $tr{'conn netstatc'});

	print "<PRE>";
	system '/bin/netstat', '-alon';
	print "</PRE>\n";

	&closebox();
}





exit;
